@@ -13,7 +13,9 @@ from typing import (
 )
 
 import astor
-from pydoctor import epydoc2stan, model
+from pydoctor import epydoc2stan, model, node2stan
+from pydoctor.epydoc.markup._pyval_repr import colorize_inline_pyval
+from pydoctor.astutils import bind_args, node2dottedname
 
 def parseFile(path: Path) -> ast.Module:
     """Parse the contents of a Python source file."""
@@ -27,19 +29,6 @@ else:
     _parse = ast.parse
 
 
-def node2dottedname(node: Optional[ast.expr]) -> Optional[List[str]]:
-    parts = []
-    while isinstance(node, ast.Attribute):
-        parts.append(node.attr)
-        node = node.value
-    if isinstance(node, ast.Name):
-        parts.append(node.id)
-    else:
-        return None
-    parts.reverse()
-    return parts
-
-
 def node2fullname(expr: Optional[ast.expr], ctx: model.Documentable) -> Optional[str]:
     dottedname = node2dottedname(expr)
     if dottedname is None:
@@ -76,21 +65,6 @@ def _handleAliasing(
     ctx._localNameToFullName_map[target] = full_name
     return True
 
-
-def bind_args(sig: Signature, call: ast.Call) -> BoundArguments:
-    """Binds the arguments of a function call to that function's signature.
-    @raise TypeError: If the arguments do not match the signature.
-    """
-    kwargs = {
-        kw.arg: kw.value
-        for kw in call.keywords
-        # When keywords are passed using '**kwargs', the 'arg' field will
-        # be None. We don't currently support keywords passed that way.
-        if kw.arg is not None
-        }
-    return sig.bind(*call.args, **kwargs)
-
-
 _attrs_decorator_signature = signature(attrs)
 """Signature of the L{attr.s} class decorator."""
 
@@ -167,6 +141,65 @@ def attrib_args(expr: ast.expr, ctx: model.Documentable) -> Optional[BoundArgume
                 )
     return None
 
+def is_using_typing_final(obj: model.Attribute) -> bool:
+    """
+    Detect if C{obj}'s L{Attribute.annotation} is using L{typing.Final}.
+    """
+    final_qualifiers = ("typing.Final", "typing_extensions.Final")
+    fullName = node2fullname(obj.annotation, obj)
+    if fullName in final_qualifiers:
+        return True
+    if isinstance(obj.annotation, ast.Subscript):
+        # Final[...] or typing.Final[...] expressions
+        if isinstance(obj.annotation.value, (ast.Name, ast.Attribute)):
+            value = obj.annotation.value
+            fullName = node2fullname(value, obj)
+            if fullName in final_qualifiers:
+                return True
+
+    return False
+
+def is_constant(obj: model.Attribute) -> bool:
+    """
+    Detect if the given assignment is a constant. 
+
+    To detect whether a assignment is a constant, this checks two things:
+        - all-caps variable name
+        - typing.Final annotation
+    
+    @note: Must be called after setting obj.annotation to detect variables using Final.
+    """
+
+    return obj.name.isupper() or is_using_typing_final(obj)
+
+def is_attribute_overridden(obj: model.Attribute, new_value: Optional[ast.expr]) -> bool:
+    """
+    Detect if the optional C{new_value} expression override the one already stored in the L{Attribute.value} attribute.
+    """
+    return obj.value is not None and new_value is not None
+
+def _extract_annotation_subscript(annotation: ast.Subscript) -> ast.AST:
+    """
+    Extract the "str, bytes" part from annotations like  "Union[str, bytes]".
+    """
+    ann_slice = annotation.slice
+    if sys.version_info < (3,9) and isinstance(ann_slice, ast.Index):
+        return ann_slice.value
+    else:
+        return ann_slice
+
+def extract_final_subscript(annotation: ast.Subscript) -> ast.expr:
+    """
+    Extract the "str" part from annotations like  "Final[str]".
+
+    @raises ValueError: If the "Final" annotation is not valid.
+    """ 
+    ann_slice = _extract_annotation_subscript(annotation)
+    if isinstance(ann_slice, (ast.ExtSlice, ast.Slice, ast.Tuple)):
+        raise ValueError("Annotation is invalid, it should not contain slices.")
+    else:
+        assert isinstance(ann_slice, ast.expr)
+        return ann_slice
 
 class ModuleVistor(ast.NodeVisitor):
     currAttr: Optional[model.Documentable]
@@ -366,6 +399,8 @@ class ModuleVistor(ast.NodeVisitor):
                             "astbuilder",
                             "moving %r into %r" % (ob.fullName(), current.fullName())
                             )
+                        # Must be a Module since the exports is set to an empty list if it's not.
+                        assert isinstance(current, model.Module)
                         ob.reparent(current, asname)
                         continue
 
@@ -426,6 +461,49 @@ class ModuleVistor(ast.NodeVisitor):
                     target_obj.kind = model.DocumentableKind.CLASS_METHOD
                 return True
         return False
+    
+    def _warnsConstantAssigmentOverride(self, obj: model.Attribute, lineno_offset: int) -> None:
+        obj.report(f'Assignment to constant "{obj.name}" overrides previous assignment '
+                    f'at line {obj.linenumber}, the original value will not be part of the docs.', 
+                            section='ast', lineno_offset=lineno_offset)
+                            
+    def _warnsConstantReAssigmentInInstance(self, obj: model.Attribute, lineno_offset: int = 0) -> None:
+        obj.report(f'Assignment to constant "{obj.name}" inside an instance is ignored, this value will not be part of the docs.', 
+                        section='ast', lineno_offset=lineno_offset)
+
+    def _handleConstant(self, obj: model.Attribute, value: Optional[ast.expr], lineno: int) -> None:
+        """Must be called after obj.setLineNumber() to have the right line number in the warning."""
+        
+        if is_attribute_overridden(obj, value):
+            
+            if obj.kind in (model.DocumentableKind.CONSTANT, 
+                                model.DocumentableKind.VARIABLE, 
+                                model.DocumentableKind.CLASS_VARIABLE):
+                # Module/Class level warning, regular override.
+                self._warnsConstantAssigmentOverride(obj=obj, lineno_offset=lineno-obj.linenumber)
+            else:
+                # Instance level warning caught at the time of the constant detection.
+                self._warnsConstantReAssigmentInInstance(obj)
+
+        obj.value = value
+        
+        obj.kind = model.DocumentableKind.CONSTANT
+
+        # A hack to to display variables annotated with Final with the real type instead.
+        if is_using_typing_final(obj):
+            if isinstance(obj.annotation, ast.Subscript):
+                try:
+                    annotation = extract_final_subscript(obj.annotation)
+                except ValueError as e:
+                    obj.report(str(e), section='ast', lineno_offset=lineno-obj.linenumber)
+                    obj.annotation = _infer_type(value) if value else None
+                else:
+                    # Will not display as "Final[str]" but rather only "str"
+                    obj.annotation = annotation
+            else:
+                # Just plain "Final" annotation.
+                # Simply ignore it because it's duplication of information.
+                obj.annotation = _infer_type(value) if value else None
 
     def _handleModuleVar(self,
             target: str,
@@ -439,14 +517,26 @@ class ModuleVistor(ast.NodeVisitor):
             return
         parent = self.builder.current
         obj = parent.resolveName(target)
+        
         if obj is None:
             obj = self.builder.addAttribute(name=target, kind=None, parent=parent)
+        
         if isinstance(obj, model.Attribute):
-            obj.kind = model.DocumentableKind.VARIABLE
+            
             if annotation is None and expr is not None:
                 annotation = _infer_type(expr)
+            
             obj.annotation = annotation
             obj.setLineNumber(lineno)
+            
+            if is_constant(obj):
+                self._handleConstant(obj=obj, value=expr, lineno=lineno)
+            else:
+                obj.kind = model.DocumentableKind.VARIABLE
+                # We store the expr value for all Attribute in order to be able to 
+                # check if they have been initialized or not.
+                obj.value = expr
+
             self.newAttr = obj
 
     def _handleAssignmentInModule(self,
@@ -470,9 +560,13 @@ class ModuleVistor(ast.NodeVisitor):
         assert isinstance(cls, model.Class)
         if not _maybeAttribute(cls, name):
             return
-        obj: Optional[model.Attribute] = cls.contents.get(name)
+
+        # Class variables can only be Attribute, so it's OK to cast
+        obj = cast(Optional[model.Attribute], cls.contents.get(name))
+
         if obj is None:
             obj = self.builder.addAttribute(name=name, kind=None, parent=cls)
+
         if obj.kind is None:
             instance = is_attrib(expr, cls) or (
                 cls.auto_attribs and annotation is not None and not (
@@ -481,13 +575,21 @@ class ModuleVistor(ast.NodeVisitor):
                     )
                 )
             obj.kind = model.DocumentableKind.INSTANCE_VARIABLE if instance else model.DocumentableKind.CLASS_VARIABLE
+
         if expr is not None:
             if annotation is None:
                 annotation = self._annotation_from_attrib(expr, cls)
             if annotation is None:
                 annotation = _infer_type(expr)
+        
         obj.annotation = annotation
         obj.setLineNumber(lineno)
+
+        if is_constant(obj):
+            self._handleConstant(obj=obj, value=expr, lineno=lineno)
+        else:
+            obj.value = expr
+
         self.newAttr = obj
 
     def _handleInstanceVar(self,
@@ -504,14 +606,26 @@ class ModuleVistor(ast.NodeVisitor):
             return
         if not _maybeAttribute(cls, name):
             return
-        obj = cls.contents.get(name)
+
+        # Class variables can only be Attribute, so it's OK to cast
+        obj = cast(Optional[model.Attribute], cls.contents.get(name))
         if obj is None:
+
             obj = self.builder.addAttribute(name=name, kind=None, parent=cls)
-        obj.kind = model.DocumentableKind.INSTANCE_VARIABLE
+
         if annotation is None and expr is not None:
             annotation = _infer_type(expr)
+        
         obj.annotation = annotation
         obj.setLineNumber(lineno)
+
+        # Maybe an instance variable overrides a constant, 
+        # so we check before setting the kind to INSTANCE_VARIABLE.
+        if obj.kind is model.DocumentableKind.CONSTANT:
+            self._warnsConstantReAssigmentInInstance(obj, lineno_offset=lineno-obj.linenumber)
+        else:
+            obj.kind = model.DocumentableKind.INSTANCE_VARIABLE
+            obj.value = expr
         self.newAttr = obj
 
     def _handleAssignmentInClass(self,
@@ -705,9 +819,9 @@ class ModuleVistor(ast.NodeVisitor):
             index -= default_offset
             return None if index < 0 else defaults[index]
 
-        parameters = []
+        parameters: List[Parameter] = []
         def add_arg(name: str, kind: Any, default: Optional[ast.expr]) -> None:
-            default_val = Parameter.empty if default is None else _ValueFormatter(default)
+            default_val = Parameter.empty if default is None else _ValueFormatter(default, ctx=func)
             parameters.append(Parameter(name, kind, default=default_val))
 
         for index, arg in enumerate(posonlyargs):
@@ -813,10 +927,12 @@ class ModuleVistor(ast.NodeVisitor):
             yield from base_args.args
             varargs = base_args.vararg
             if varargs:
+                varargs.arg = epydoc2stan.VariableArgument(varargs.arg)
                 yield varargs
             yield from base_args.kwonlyargs
             kwargs = base_args.kwarg
             if kwargs:
+                kwargs.arg = epydoc2stan.KeywordArgument(kwargs.arg)
                 yield kwargs
         def _get_all_ast_annotations() -> Iterator[Tuple[str, Optional[ast.expr]]]:
             for arg in _get_all_args():
@@ -848,32 +964,31 @@ class ModuleVistor(ast.NodeVisitor):
             assert isinstance(expr, ast.expr), expr
             return expr
 
-
 class _ValueFormatter:
-    """Formats values stored in AST expressions.
+    """
+    Class to encapsulate a python value and translate it to HTML when calling L{repr()} on the L{_ValueFormatter}.
     Used for presenting default values of parameters.
     """
 
-    def __init__(self, value: ast.expr):
-        self.value = value
+    def __init__(self, value: Any, ctx: model.Documentable):
+        self._colorized = colorize_inline_pyval(value)
+        """
+        The colorized value as L{ParsedDocstring}.
+        """
+
+        self._linker = epydoc2stan._EpydocLinker(ctx)
+        """
+        Linker.
+        """
 
     def __repr__(self) -> str:
-        value = self.value
-        if isinstance(value, ast.Num):
-            return str(value.n)
-        if isinstance(value, ast.Str):
-            return repr(value.s)
-        if isinstance(value, ast.Constant):
-            return repr(value.value)
-        if isinstance(value, ast.UnaryOp) and isinstance(value.op, ast.USub):
-            operand = value.operand
-            if isinstance(operand, ast.Num):
-                return f'-{operand.n}'
-            if isinstance(operand, ast.Constant):
-                return f'-{operand.value}'
-        source: str = astor.to_source(value)
-        return source.strip()
-
+        """
+        Present the python value as HTML. 
+        Without the englobing <code> tags.
+        """
+        # Using node2stan.node2html instead of flatten(to_stan()). 
+        # This avoids calling flatten() twice.
+        return ''.join(node2stan.node2html(self._colorized.to_node(), self._linker))
 
 class _AnnotationStringParser(ast.NodeTransformer):
     """Implementation of L{ModuleVistor._unstring_annotation()}.
@@ -931,7 +1046,6 @@ def _infer_type(expr: ast.expr) -> Optional[ast.expr]:
     @param expr: The expression's AST.
     @return: A type annotation, or None if the expression has no obvious type.
     """
-
     try:
         value: object = ast.literal_eval(expr)
     except ValueError:
