@@ -43,12 +43,12 @@ import sre_constants
 from inspect import signature
 from typing import Any, AnyStr, Union, Callable, Dict, Iterable, Sequence, Optional, List, Tuple, cast
 
-import sre_parse36 #https://github.com/tristanlatr/sre_parse36
 import attr
 import astor.op_util
 from docutils import nodes, utils
 from twisted.web.template import Tag
 
+from pydoctor.epydoc import sre_parse36
 from pydoctor.epydoc.markup import DocstringLinker
 from pydoctor.epydoc.markup.restructuredtext import ParsedRstDocstring
 from pydoctor.epydoc.docutils import set_node_attributes, wbr, obj_reference
