@@ -88,6 +88,7 @@ Description::
    <!ATTLIST code style CDATA #IMPLIED>
 
    <!ELEMENT symbol (#PCDATA)>
+   <!ELEMENT wbr>
 
 @var SYMBOLS: A list of the of escape symbols that are supported by epydoc.  Currently the following symbols are supported ::
 
@@ -131,14 +132,16 @@ __docformat__ = 'epytext en'
 #   4. helpers
 #   5. testing
 
-from typing import Any, Iterable, List, Optional, Sequence, Union, cast, overload
+from typing import Any, Callable, Iterable, List, Optional, Sequence, Union, cast, overload
 import re
 
 from docutils import utils, nodes
 from twisted.web.template import Tag
 
-from pydoctor.epydoc.docutils import set_node_attributes
 from pydoctor.epydoc.markup import Field, ParseError, ParsedDocstring
+from pydoctor.epydoc.markup._types import ParsedTypeDocstring
+from pydoctor.epydoc.docutils import set_node_attributes
+from pydoctor.model import Documentable
 
 ##################################################
 ## DOM-Like Encoding
@@ -1232,7 +1235,7 @@ class ColorizingError(ParseError):
 ##                    SUPPORT FOR EPYDOC
 #################################################################
 
-def parse_docstring(docstring: str, errors: List[ParseError]) -> ParsedDocstring:
+def parse_docstring(docstring: str, errors: List[ParseError], processtypes: bool = False) -> ParsedDocstring:
     """
     Parse the given docstring, which is formatted using epytext; and
     return a L{ParsedDocstring} representation of its contents.
@@ -1240,6 +1243,7 @@ def parse_docstring(docstring: str, errors: List[ParseError]) -> ParsedDocstring
     @param docstring: The docstring to parse
     @param errors: A list where any errors generated during parsing
         will be stored.
+    @param processtypes: Use L{ParsedTypeDocstring} to parsed 'type' fields.
     """
     tree = parse(docstring, errors)
     if tree is None:
@@ -1266,9 +1270,18 @@ def parse_docstring(docstring: str, errors: List[ParseError]) -> ParsedDocstring
 
             # Process the field.
             field.tag = 'epytext'
-            fieldDoc = ParsedEpytextDocstring(field, ())
+
+            field_parsed_doc: ParsedDocstring = ParsedEpytextDocstring(field, ())
+
             lineno = int(field.attribs['lineno'])
-            fields.append(Field(tag, arg, fieldDoc, lineno))
+            
+            # This allows epytext markup to use TypeDocstring as well with a CLI option: --process-types
+            if processtypes and tag in ParsedTypeDocstring.FIELDS:
+                field_parsed_doc = ParsedTypeDocstring(field_parsed_doc.to_node(), lineno=lineno)
+                for warning_msg in field_parsed_doc.warnings:
+                    errors.append(ParseError(warning_msg, lineno, is_fatal=False))
+            
+            fields.append(Field(tag, arg, field_parsed_doc, lineno))
 
     # Save the remaining docstring as the description.
     if tree_children and tree_children[0].children:
@@ -1276,6 +1289,11 @@ def parse_docstring(docstring: str, errors: List[ParseError]) -> ParsedDocstring
     else:
         return ParsedEpytextDocstring(None, fields)
 
+def get_parser(obj: Optional[Documentable]) -> Callable[[str, List[ParseError], bool], ParsedDocstring]:
+    """
+    Get the L{parse_docstring} function. 
+    """
+    return parse_docstring
 
 class ParsedEpytextDocstring(ParsedDocstring):
     SYMBOL_TO_CODEPOINT = {
