@@ -17,7 +17,7 @@ for the C{docutils.nodes.document} class.
 
 B{Creating C{ParsedRstDocstring}s}:
 
-C{ParsedRstDocstring}s are created by the C{parse_document} function,
+C{ParsedRstDocstring}s are created by the L{parse_docstring} function,
 using the C{docutils.core.publish_string()} method, with the following
 helpers:
 
@@ -41,18 +41,22 @@ the list.
 """
 __docformat__ = 'epytext en'
 
-from typing import Iterable, List, Optional, Sequence, Set
+from typing import Callable, Iterable, List, Optional, Sequence, Set, cast
+import re
+from docutils import nodes
 
 from docutils.core import publish_string
 from docutils.writers import Writer
+from docutils.parsers.rst.directives.admonitions import BaseAdmonition # type: ignore[import]
 from docutils.readers.standalone import Reader as StandaloneReader
 from docutils.utils import Reporter, new_document
 from docutils.parsers.rst import Directive, directives #type: ignore[attr-defined]
 from docutils.transforms import Transform, frontmatter
-from docutils import nodes
 
 from pydoctor.epydoc.markup import Field, ParseError, ParsedDocstring
 from pydoctor.epydoc.markup.plaintext import ParsedPlaintextDocstring
+from pydoctor.epydoc.markup._types import ParsedTypeDocstring
+from pydoctor.model import Documentable
 
 #: A dictionary whose keys are the "consolidated fields" that are
 #: recognized by epydoc; and whose values are the corresponding epydoc
@@ -75,7 +79,7 @@ CONSOLIDATED_FIELDS = {
 #: a @type field.
 CONSOLIDATED_DEFLIST_FIELDS = ['param', 'arg', 'var', 'ivar', 'cvar', 'keyword']
 
-def parse_docstring(docstring: str, errors: List[ParseError]) -> ParsedDocstring:
+def parse_docstring(docstring: str, errors: List[ParseError], processtypes: bool = False) -> ParsedDocstring:
     """
     Parse the given docstring, which is formatted using
     ReStructuredText; and return a L{ParsedDocstring} representation
@@ -84,20 +88,34 @@ def parse_docstring(docstring: str, errors: List[ParseError]) -> ParsedDocstring
     @param docstring: The docstring to parse
     @param errors: A list where any errors generated during parsing
         will be stored.
+    @param processtypes: Use L{ParsedTypeDocstring} to parsed 'type' fields.
     """
     writer = _DocumentPseudoWriter()
     reader = _EpydocReader(errors) # Outputs errors to the list.
+
+    # Credits: mhils - Maximilian Hils from the pdoc repository https://github.com/mitmproxy/pdoc
+    # Strip Sphinx interpreted text roles for code references: :obj:`foo` -> `foo`
+    docstring = re.sub(
+        r"(:py)?:(mod|func|data|const|class|meth|attr|exc|obj):", "", docstring
+    )
+
     publish_string(docstring, writer=writer, reader=reader,
                    settings_overrides={'report_level':10000,
                                        'halt_level':10000,
                                        'warning_stream':None})
 
     document = writer.document
-    visitor = _SplitFieldsTranslator(document, errors)
+    visitor = _SplitFieldsTranslator(document, errors, processtypes=processtypes)
     document.walk(visitor)
 
     return ParsedRstDocstring(document, visitor.fields)
 
+def get_parser(obj:Documentable) -> Callable[[str, List[ParseError], bool], ParsedDocstring]:
+    """
+    Get the L{parse_docstring} function. 
+    """
+    return parse_docstring
+
 class OptimizedReporter(Reporter):
     """A reporter that ignores all debug messages.  This is used to
     shave a couple seconds off of epydoc's run time, since docutils
@@ -139,7 +157,7 @@ class ParsedRstDocstring(ParsedDocstring):
 class _EpydocReader(StandaloneReader):
     """
     A reader that captures all errors that are generated by parsing,
-    and appends them to a list.
+    and appends them to a list as L{ParseError}.
     """
 
     def __init__(self, errors: List[ParseError]):
@@ -200,11 +218,12 @@ class _SplitFieldsTranslator(nodes.NodeVisitor):
     consolidated fields expressed as unordered lists still require
     backticks for now."""
 
-    def __init__(self, document: nodes.document, errors: List[ParseError]):
+    def __init__(self, document: nodes.document, errors: List[ParseError], processtypes: bool = False):
         nodes.NodeVisitor.__init__(self, document)
         self._errors = errors
         self.fields: List[Field] = []
         self._newfields: Set[str] = set()
+        self._processtypes = processtypes
 
     def visit_document(self, node: nodes.Node) -> None:
         self.fields = []
@@ -214,10 +233,15 @@ class _SplitFieldsTranslator(nodes.NodeVisitor):
         node.parent.remove(node)
 
         # Extract the field name & optional argument
+        # FIXME: https://github.com/twisted/pydoctor/issues/267
+        #   Support combined parameter type and description, if the type is a single word like::
+        #       :param str user_agent: user agent
         tag = node[0].astext().split(None, 1)
         tagname = tag[0]
-        if len(tag)>1: arg = tag[1]
-        else: arg = None
+        if len(tag)>1: 
+            arg = tag[1]
+        else: 
+            arg = None
 
         # Handle special fields:
         fbody = node[1]
@@ -250,9 +274,18 @@ class _SplitFieldsTranslator(nodes.NodeVisitor):
             lineno: int
             ) -> None:
         field_doc = self.document.copy()
-        for child in fbody: field_doc.append(child)
-        field_pdoc = ParsedRstDocstring(field_doc, ())
-        self.fields.append(Field(tagname, arg, field_pdoc, lineno - 1))
+        for child in fbody: 
+            field_doc.append(child)
+
+        # This allows restructuredtext markup to use TypeDocstring as well with a CLI option: --process-types
+        field_parsed_doc: ParsedDocstring
+        if self._processtypes and tagname in ParsedTypeDocstring.FIELDS:
+            field_parsed_doc = ParsedTypeDocstring(field_doc)
+            for warning_msg in field_parsed_doc.warnings:
+                    self._errors.append(ParseError(warning_msg, lineno, is_fatal=False))
+        else:
+            field_parsed_doc = ParsedRstDocstring(field_doc, ())
+        self.fields.append(Field(tagname, arg, field_parsed_doc, lineno - 1))
 
     def visit_field_list(self, node: nodes.Node) -> None:
         # Remove the field list from the tree.  The visitor will still walk
@@ -364,6 +397,79 @@ class _SplitFieldsTranslator(nodes.NodeVisitor):
     def unknown_visit(self, node: nodes.Node) -> None:
         'Ignore all unknown nodes'
 
+versionlabels = {
+    'versionadded':   'New in version %s',
+    'versionchanged': 'Changed in version %s',
+    'deprecated':     'Deprecated since version %s',
+}
+
+versionlabel_classes = {
+    'versionadded':     'added',
+    'versionchanged':   'changed',
+    'deprecated':       'deprecated',
+}
+
+class VersionChange(Directive):
+    """
+    Directive to describe a change/addition/deprecation in a specific version.
+    """
+    class versionmodified(nodes.Admonition, nodes.TextElement):
+        """Node for version change entries.
+        Currently used for "versionadded", "versionchanged" and "deprecated"
+        directives.
+        """
+    
+    has_content = True
+    required_arguments = 1
+    optional_arguments = 1
+    final_argument_whitespace = True
+
+    def run(self) -> List[nodes.Node]:
+        node = self.versionmodified()
+        node.document = self.state.document
+        node['type'] = self.name
+        node['version'] = self.arguments[0]
+        text = versionlabels[self.name] % self.arguments[0]
+        if len(self.arguments) == 2:
+            inodes, messages = self.state.inline_text(self.arguments[1],
+                                                      self.lineno + 1)
+            para = nodes.paragraph(self.arguments[1], '', *inodes)
+            node.append(para)
+        else:
+            messages = []
+        if self.content:
+            self.state.nested_parse(self.content, self.content_offset, node)
+        classes = ['versionmodified', versionlabel_classes[self.name]]
+        if len(node):
+            if isinstance(node[0], nodes.paragraph) and node[0].rawsource:
+                content = nodes.inline(node[0].rawsource)
+                content.source = node[0].source
+                content.line = node[0].line
+                content += node[0].children
+                node[0].replace_self(nodes.paragraph('', '', content))
+
+            para = cast(nodes.paragraph, node[0])
+            para.insert(0, nodes.inline('', '%s: ' % text, classes=classes))
+        else:
+            para = nodes.paragraph('', '',
+                                   nodes.inline('', '%s.' % text,
+                                                classes=classes), )
+            node.append(para)
+
+        ret = [node]  # type: List[nodes.Node]
+        ret += messages
+        return ret
+
+# Do like Sphinx does for the seealso directive. 
+class SeeAlso(BaseAdmonition):
+    """
+    An admonition mentioning things to look at as reference.
+    """
+    class seealso(nodes.Admonition, nodes.Element):
+        """Custom "see also" admonition node."""
+
+    node_class = seealso
+
 class PythonCodeDirective(Directive):
     """
     A custom restructuredtext directive which can be used to display
@@ -383,3 +489,7 @@ class PythonCodeDirective(Directive):
         return [ node ]
 
 directives.register_directive('python', PythonCodeDirective)
+directives.register_directive('versionadded', VersionChange)
+directives.register_directive('versionchanged', VersionChange)
+directives.register_directive('deprecated', VersionChange)
+directives.register_directive('seealso', SeeAlso)
