@@ -18,7 +18,7 @@ from inspect import Signature
 from optparse import Values
 from pathlib import Path
 from typing import (
-    TYPE_CHECKING, Any, Collection, Dict, Iterable, Iterator, List, Mapping,
+    TYPE_CHECKING, Collection, Dict, Iterable, Iterator, List, Mapping,
     Optional, Sequence, Set, Tuple, Type, TypeVar, Union, overload
 )
 from urllib.parse import quote
@@ -95,6 +95,7 @@ class DocumentableKind(Enum):
     STATIC_METHOD       = 600
     METHOD              = 500
     FUNCTION            = 400
+    CONSTANT            = 310
     CLASS_VARIABLE      = 300
     SCHEMA_FIELD        = 220
     ATTRIBUTE           = 210
@@ -142,9 +143,7 @@ class Documentable:
         return self
 
     def setup(self) -> None:
-        # TODO: The actual value type is Documentable, but using that
-        #       requires a boatload of changes.
-        self.contents: Dict[str, Any] = {}
+        self.contents: Dict[str, Documentable] = {}
 
     def setDocstring(self, node: ast.Str) -> None:
         doc = node.s
@@ -525,7 +524,12 @@ class Attribute(Inheritable):
     kind: Optional[DocumentableKind] = DocumentableKind.ATTRIBUTE
     annotation: Optional[ast.expr]
     decorators: Optional[Sequence[ast.expr]] = None
+    value: Optional[ast.expr] = None
+    """
+    The value of the assignment expression. 
 
+    None value means the value is not initialized at the current point of the the process. 
+    """
 
 # Work around the attributes of the same name within the System class.
 _ModuleT = Module
