@@ -14,6 +14,7 @@ from pydoctor.templatewriter.pages import Page
 
 if TYPE_CHECKING:
     from twisted.web.template import Flattenable
+    from typing_extensions import Final
 
 
 def moduleSummary(module: model.Module, page_url: str) -> Tag:
@@ -347,7 +348,7 @@ class UndocumentedSummaryPage(Page):
                 ))
         return tag
 
-summarypages: Iterable[Type[Page]] = [
+summarypages: 'Final[Iterable[Type[Page]]]' = [
     ModuleIndexPage,
     ClassIndexPage,
     IndexPage,
