@@ -1,8 +1,8 @@
 from typing import List
 
-from pydoctor.epydoc.markup import DocstringLinker, ParseError, epytext, flatten
+from pydoctor.epydoc.markup import DocstringLinker, ParseError, epytext
 from pydoctor.test import NotFoundLinker
-
+from pydoctor.stanutils import flatten
 
 def epytext2html(s: str, linker: DocstringLinker = NotFoundLinker()) -> str:
     errs: List[ParseError] = []
@@ -25,10 +25,10 @@ def test_basic_list() -> None:
     LI3 = "  - This is a list\n  item."
     LI4 = "\n  - This is a list\n  item."
     PARA = ('<para>This is a paragraph.</para>')
-    ONELIST = ('<ulist><li><para inline=True>This is a '
+    ONELIST = ('<ulist><li><para>This is a '
                'list item.</para></li></ulist>')
-    TWOLIST = ('<ulist><li><para inline=True>This is a '
-               'list item.</para></li><li><para inline=True>This is a '
+    TWOLIST = ('<ulist><li><para>This is a '
+               'list item.</para></li><li><para>This is a '
                'list item.</para></li></ulist>')
 
     for p in (P1, P2):
@@ -45,7 +45,7 @@ def test_basic_list() -> None:
                 assert parse(f'{p}\n{li1}\n{li2}\n{p}') == PARA+TWOLIST+PARA
 
     LI5 = "  - This is a list item.\n\n    It contains two paragraphs."
-    LI5LIST = ('<ulist><li><para inline=True>This is a list item.</para>'
+    LI5LIST = ('<ulist><li><para>This is a list item.</para>'
                '<para>It contains two paragraphs.</para></li></ulist>')
     assert parse(LI5) == LI5LIST
     assert parse(f'{P1}\n{LI5}') == PARA+LI5LIST
@@ -53,7 +53,7 @@ def test_basic_list() -> None:
 
     LI6 = ("  - This is a list item with a literal block::\n"
            "    hello\n      there")
-    LI6LIST = ('<ulist><li><para inline=True>This is a list item with a literal '
+    LI6LIST = ('<ulist><li><para>This is a list item with a literal '
                'block:</para><literalblock>  hello\n    there'
                '</literalblock></li></ulist>')
     assert parse(LI6) == LI6LIST
@@ -63,10 +63,10 @@ def test_basic_list() -> None:
 
 def test_item_wrap() -> None:
     LI = "- This is a list\n  item."
-    ONELIST = ('<ulist><li><para inline=True>This is a '
+    ONELIST = ('<ulist><li><para>This is a '
                'list item.</para></li></ulist>')
-    TWOLIST = ('<ulist><li><para inline=True>This is a '
-               'list item.</para></li><li><para inline=True>This is a '
+    TWOLIST = ('<ulist><li><para>This is a '
+               'list item.</para></li><li><para>This is a '
                'list item.</para></li></ulist>')
     for indent in ('', '  '):
         for nl1 in ('', '\n'):
@@ -79,8 +79,8 @@ def test_literal_braces() -> None:
     """SF bug #1562530 reported some trouble with literal braces.
     This test makes sure that braces are getting rendered as desired.
     """
-    assert epytext2html("{1:{2:3}}") == '<p>{1:{2:3}}</p>'
-    assert epytext2html("C{{1:{2:3}}}") == '<p><code>{1:{2:3}}</code></p>'
-    assert epytext2html("{1:C{{2:3}}}") == '<p>{1:<code>{2:3}</code>}</p>'
-    assert epytext2html("{{{}{}}{}}") == '<p>{{{}{}}{}}</p>'
-    assert epytext2html("{{E{lb}E{lb}E{lb}}}") == '<p>{{{{{}}</p>'
+    assert epytext2html("{1:{2:3}}") == '{1:{2:3}}'
+    assert epytext2html("C{{1:{2:3}}}") == '<tt class="rst-docutils literal"><span class="pre">{1:{2:3}}</span></tt>'
+    assert epytext2html("{1:C{{2:3}}}") == '{1:<tt class="rst-docutils literal">{2:3}</tt>}'
+    assert epytext2html("{{{}{}}{}}") == '{{{}{}}{}}'
+    assert epytext2html("{{E{lb}E{lb}E{lb}}}") == '{{{{{}}'
